unit uGoods;

interface

uses SysUtils, Classes, DXSprite,DXDraws,uConst,Dialogs,
  extctrls,forms,uSynergy,HidControllerClass;

type
  TNoteRec=record
    bar:Integer;
    beat:Integer;
    tick:Integer;
  end;

type
  TBaseObject = class (TObject)
  end;
  
  TExtrinsic = class (TImageSprite)
  public
    procedure StandOn(vX,vY:Double);
  end;
  
  TIntrinsic = class (TBaseObject)
  end;
  
  TActive = class (TExtrinsic)
  private
    FTimer: TTimer;
    procedure DoTimerMove(Sender:TObject); virtual;
    function GetFrameRate: Integer;
    procedure SetFrameRate(const Value: Integer);
  public
    constructor Create(AParent:TSprite); override;
    destructor Destroy; override;
    procedure Run;
    procedure Stop;
    property FrameRate: Integer read GetFrameRate write SetFrameRate;
  end;
  
  TPrize = class (TActive)
  protected
    FDead: Boolean;
    FDeadMovePixel: Integer;
    FOverline: Boolean;
    FTag: Integer;
  public
    constructor Create(AParent:TSprite); override;
    procedure Move(Step:Integer);
    procedure Overline;
    procedure ToDead;
    property Tag: Integer read FTag write FTag;
  end;
  
  {1 sy }
  TChainBall = class (TPrize)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  {1 Ʋy }
  TColorBall = class (TPrize)
  end;
  
  {1 ⪺pyAݭn }
  TYellowBall = class (TColorBall)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  TBlueBall = class (TColorBall)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  {1 ⪺jyAݭn }
  TRedBall = class (TColorBall)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  {1 êy }
  TFeces = class (TPrize)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  TInactive = class (TExtrinsic)
  end;
  
  TLiningType=(ltRed,ltYellow,ltBlue);
  {1  }
  TGesture = class (TInactive)
  private
    procedure SetValue(Value: Integer);
  public
    constructor Create(AParent:TSprite); override;
    property Value: Integer write SetValue;
  end;
  
  {1 A }
  TLiningGesture = class (TInactive)
  private
    FStatus: TLiningType;
    procedure SetStaus(const Value: TLiningType);
    procedure SetValue(Value: Integer);
  public
    constructor Create(AParent:TSprite); override;
    property Status: TLiningType read FStatus write SetStaus;
    property Value: Integer write SetValue;
  end;
  
  {1 Dy }
  TThwart = class (TPrize)
  end;
  
  {1 lly }
  THammer = class (TThwart)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  {1 ƤƦr }
  TScoreNum = class (TInactive)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  {1  }
  TMarker = class (TBaseObject)
  private
    FDigits: array[0..5] of TScoreNum;
    FNoteCount: Integer;
    FValue: Integer;
    procedure Display;
    procedure SetRole(const Value: Integer);
  public
    constructor Create;
    procedure Inc(Value:Integer);
    procedure IncNote;
    procedure Reset;
    property NoteCount: Integer read FNoteCount;
    property Role: Integer write SetRole;
  end;
  
  TMelodyProducer = class (TIntrinsic)
  end;
  
  {1 Ԫ̲y }
  TSmog = class (TThwart)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  {1 I }
  TBackScenery = class (TActive)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  TBackSceneryOne = class (TBackScenery)
  public
    constructor Create(AParent:TSprite); override;
    procedure DoTimerMove(Sender:TObject); override;
  end;
  
  TBackSceneryTwo = class (TBackScenery)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  {1 Dʵe }
  TPrincipal = class (TActive)
  public
    constructor Create(AParent:TSprite;AnimName:String);
    procedure StandOn(vX,vY:Integer);
  end;
  
  {1 eʵeASĪ{ΦH }
  TWebbitToy = class (TActive)
  private
    procedure DoTimerMove(Sender:TObject); override;
  public
    constructor Create(AParent:TSprite;AnimName:String);
  end;
  
  {1 e{ʵe }
  TAcrobatics = class (TBaseObject)
  private
    FPrincipal: TPrincipal;
    FShowcase: array of TWebbitToy;
    FWallcloth: TBackScenery;
  public
    constructor Create;virtual;
    procedure Go; virtual;
    procedure Stay; virtual;
  end;

  TAcrobaticsOne = class (TAcrobatics)
  public
    constructor Create;override;
    destructor Destroy; override;
    procedure Go; override;
    procedure Stay; override;
  end;

  TAcrobaticsTwo = class (TAcrobatics)
  public
    constructor Create;override;
    destructor Destroy; override;
    procedure Go; override;
    procedure Stay; override;
  end;
  
  TPrecise = class (TActive)
  private
    FTickCount: Integer;
    procedure SetEnabled(const Value: Boolean);
    procedure SetRole(const Value: Integer); virtual; abstract;
  public
    constructor Create(AParent:TSprite); override;
    property Enabled: Boolean write SetEnabled;
    property Role: Integer write SetRole;
  end;
  
  {1 دS }
  TExplode = class (TPrecise)
  public
    constructor Create(AParent:TSprite); override;
    procedure DoTimerMove(Sender:TObject); override;
    procedure SetRole(const Value:Integer); override;
  end;
  
  {1 OKS }
  TOk = class (TPrecise)
  public
    constructor Create(AParent:TSprite); override;
    procedure DoTimerMove(Sender:TObject); override;
    procedure SetRole(const Value:Integer); override;
  end;
  
  TRule = class (TInactive)
  end;
  
  TRuleMark = class (TRule)
  private
    procedure SetRole(const Value: Integer);
  public
    constructor Create(AParent:TSprite); override;
    procedure DoCollision(Sprite:TSprite;var Done:Boolean); override;
    procedure DoMove(MoveCount:Integer); override;
    property Role: Integer write SetRole;
  end;
  
  {1 a1 }
  TRule1P = class (TRule)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  {1 a2 }
  TRule2P = class (TRule)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  TComboNum = class (TInactive)
  private
    FDigit: Integer;
    procedure SetDigit(const Value: Integer);
  public
    constructor Create(AParent:TSprite); override;
    property Digit: Integer read FDigit write SetDigit;
  end;
  
  TComboValue = class (TBaseObject)
  private
    FDigitTen: TComboNum;
    FDigitUnit: TComboNum;
    function GetValue: Integer;
    procedure SetValue(Value: Integer);
    procedure SetVisible(const Value: Boolean);
  public
    constructor Create;
    procedure StandOn(vX,vY:Integer);
    property Value: Integer read GetValue write SetValue;
    property Visible: Boolean write SetVisible;
  end;
  
  TComboImg = class (TInactive)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  TCombo = class (TBaseObject)
  private
    FTitle: TComboImg;
    FValue: TComboValue;
    function GetValue: Integer;
    procedure SetRole(const Value: Integer);
    procedure SetValue(const Value: Integer);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    procedure Inc;
    property Role: Integer write SetRole;
    property Value: Integer read GetValue write SetValue;
  end;
  
  {1 ʤƦr }
  TPercentNum = class (TActive)
  private
    FDigit: Integer;
    procedure SetDigit(const Value: Integer);
  public
    constructor Create(AParent:TSprite); override;
    property Digit: Integer read FDigit write SetDigit;
  end;
  
  {1 ʤŸ }
  TPercentSign = class (TInactive)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  {1 qXv }
  TPercentag = class (TBaseObject)
  private
    FDigitTen: TPercentNum;
    FDigitUnit: TPercentNum;
    FSign: TPercentSign;
    procedure SetRole(const Value: Integer);
    procedure SetValue(const Value: Integer);
  public
    constructor Create;
    property Role: Integer write SetRole;
    property Value: Integer write SetValue;
  end;
  
  {1 qW }
  TSongName = class (TInactive)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  TPrizeKind=(pkBlue,pkYellow,pkRed,pkChain,pkLoo,pkHammer,pkSmog);
  TBackPipeLine = class (TInactive)
  end;
  
  TBackPipeLineTwo = class (TBackPipeLine)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  TBackPipeLineOne = class (TBackPipeLine)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  TStepMark = class (TInactive)
  public
    constructor Create(AParent:TSprite;ImgIndex:Integer);
  end;
  
  TStressValue = class (TBaseObject)
  private
    FCount: Integer;
    FEasyMode: Boolean;
    FHidDev: TJvHidDevice;
    FLastValue: Integer;
    FNeedRelease: Boolean;
    FRelease: Boolean;
    FValue: Integer;
    procedure SetValue(Value: Integer); virtual; abstract;
  public
    constructor Create;
    procedure Refresh;
    property EasyMode: Boolean read FEasyMode write FEasyMode;
    property HidDev: TJvHidDevice read FHidDev write FHidDev;
    property Value: Integer read FValue write SetValue;
  end;
  
  {1 Oװ }
  TStressValueOne = class (TStressValue)
  private
    FStepMarks: array [0..5] of TStepMark;
    procedure SetValue(Value:Integer); override;
  public
    constructor Create;
    destructor Destroy; override;
  end;

  TStressValueTwo = class (TStressValue)
  private
    FStepMarks: array [0..5] of TStepMark;
  public
    constructor Create;
    destructor Destroy; override;
  end;
  
  TMeter = class (TInactive)
  end;
  
  {1 Ȫ }
  TMeter1P = class (TMeter)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  {1 Ȫ }
  TMeter2P = class (TMeter)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  {1 Ae }
  TToyArise = class (TBaseObject)
  private
    FGesture: TGesture;
    FLiningGesture: TLiningGesture;
    FMeter: TMeter;
    FStressValue: TStressValue;
    procedure SetRole(const Value: Integer);
    procedure SetValue(const Value: Integer);
  public
    constructor Create;
    destructor Destroy; override;
    property Role: Integer write SetRole;
    property Value: Integer write SetValue;
  end;

  TPipeLine = class (TBaseObject)
  private
    FBackPipeLine: TBackPipeLine;
    FNotes: array of TPrize;
    FRule: TRule;
    FRuleMark: TRuleMark;
  protected
    function CreatePrize(PrizeKind:TPrizeKind): TPrize;
    procedure SetBringBalls(Spread: TBits);virtual;abstract;
  public
    constructor Create;virtual;
    destructor Destroy; override;
    procedure DoDraw; virtual; abstract;
    procedure Go; virtual; abstract;
    {1 ͥXsy }
    procedure NewPrize(PrizeKind:TPrizeKind); virtual;
    procedure Shift(Steps:Integer);
    procedure StepOne;
    procedure TurnLive(Index:Integer);
    property Spread: TBits write SetBringBalls;
  end;

  {1 eye }
  TPipeLineOne = class (TPipeLine)
  public
    constructor Create; override;
    destructor Destroy; override;
    procedure Go; override;
    procedure Move(Step:Integer);
    procedure SetBringBalls(Spread: TBits);override;
  end;

  TPipeLineTwo = class (TPipeLine)
  public
    constructor Create; override;
    destructor Destroy; override;
    procedure Go; override;
    procedure Move(Step:Integer);
    procedure SetBringBalls(Spread: TBits);override;
  end;

  {1 \X{yϰ }
  TSunshadeLeft = class (TInactive)
  public
    constructor Create(AParent:TSprite); override;
  end;

  {1 \kX{yϰ }
  TSunshadeRight = class (TInactive)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  {1 qXv }
  TQualifiedGoal = class (TInactive)
  public
    constructor Create(AParent:TSprite); override;
  end;
  
  TPlayerScreen = class (TBaseObject)
  private
    FAcrobatics: TAcrobatics;
    FCombo: TCombo;
    FMarker: TMarker;
    FPercentag: TPercentag;
    FPipeLine: TPipeLine;
    FSunshadeLeft: TSunshadeLeft;
    FSunshadeRight: TSunshadeRight;
    FToyArise: TToyArise;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Go;
    procedure NewBall;
    procedure Stay;
  end;
  
  TPlayerOneScreen = class (TPlayerScreen)
  private
    FQualifiedGoal: TQualifiedGoal;
    FSongName: TSongName;
  public
    constructor Create; override;
    destructor Destroy; override;
  end;

  TPlayerTwoScreen = class (TPlayerScreen)
  public
    constructor Create; override;
    destructor Destroy; override;
  end;

  {1 ӹC }
  TGripGame = class (TBaseObject)
  private
    FMetronome: TMetronome;
    FPlayer1: TPlayerScreen;
    FPlayer2: TPlayerScreen;
    FRocker: TRocker;
    function GetComboOne: TCombo;
    function GetMakerOne: TMarker;
    function GetPercentagOne: TPercentag;
    function GetStressOne: TStressValue;
    function GetStressTwo: TStressValue;
  public
    constructor Create;
    destructor Destroy; override;
    procedure ForTest;
    procedure NotifyDevice;
    procedure NotifyNote(Note:TNoteRec);
    procedure Restart;
    procedure Resume;
    procedure Stop;
    property ComboOne: TCombo read GetComboOne;
    property MarkerOne: TMarker read GetMakerOne;
    property PercentagOne: TPercentag read GetPercentagOne;
    property StressOne: TStressValue read GetStressOne;
    property StressTwo: TStressValue read GetStressTwo;
  end;
  
  procedure CreateGoodsObjects;

implementation

uses uGlobal, uDatas, uScreen,mmsystem;

procedure CreateGoodsObjects;
begin
end;

{
******************************* TBackSceneryTwo ********************************
}
constructor TBackSceneryTwo.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreFixed.Items.Find('Back2P');
  Width:=Image.Width;
  Height:=Image.Height;
  X:=33;
  Y:=178;
end;

{
********************************** TPrincipal **********************************
}
constructor TPrincipal.Create(AParent:TSprite;AnimName:String);
begin
  inherited Create(AParent);
  Image:=ImgStoreAnimation.Items.Find(AnimName);
  Width:=Image.Width;
  Height:=Image.Height;
end;

procedure TPrincipal.StandOn(vX,vY:Integer);
begin
  X:=vX;
  Y:=vY;
  AnimCount:=6;
  AnimLooped:=True;
  AnimSpeed:=1;
  AnimPos:=0;
end;

{
********************************** TWebbitToy **********************************
}
constructor TWebbitToy.Create(AParent:TSprite;AnimName:String);
begin
  inherited Create(AParent);
  Image:=ImgStoreAnimation.Items.Find(AnimName);
  Width:=Image.Width;
  Height:=Image.Height;
  if AnimName=ImgAnimationCubeName then
    AnimCount:=6;
  if AnimName=ImgAnimationCylinderName then
    AnimCount:=6;
  if AnimName=ImgAnimationTomatoName then
    AnimCount:=9;
  AnimPos:=0;
end;

procedure TWebbitToy.DoTimerMove(Sender:TObject);
begin
  AnimPos:=AnimPos+1;
  if AnimPos>=AnimCount then AnimPos:=0;
end;

{
********************************* TAcrobatics **********************************
}
constructor TAcrobatics.Create;
begin
  inherited Create;
  
  SetLength(FShowcase,3);
end;

procedure TAcrobatics.Go;
var
  i: Integer;
begin
  FPrincipal.Run;
  FWallcloth.Run;
  for i:=0 to 2 do
    FShowcase[i].Run;
end;

procedure TAcrobatics.Stay;
var
  i: Integer;
begin
  FPrincipal.Stop;
  for i:=0 to 2 do
    FShowcase[i].Stop;
end;

{
******************************** TAcrobaticsOne ********************************
}
constructor TAcrobaticsOne.Create;
var
  i: Integer;
begin
  inherited Create;

  FWallcloth:=TBackSceneryOne.Create(LayoutTwo.Engine);
  FPrincipal:=TPrincipal.Create(LayoutThree.Engine,
                  ImgAnimationJumpPleyerOneName);
  FPrincipal.FrameRate:=4;
  FPrincipal.StandOn(30,-3);
  FPrincipal.Stop;
  FShowcase[0]:=TWebbitToy.Create(LayoutThree.Engine,
        ImgAnimationCubeName);
  FShowcase[0].StandOn(115,0);
  FShowcase[1]:=TWebbitToy.Create(LayoutThree.Engine,
        ImgAnimationCylinderName);
  FShowcase[1].StandOn(135,-20);
  FShowcase[2]:=TWebbitToy.Create(LayoutThree.Engine,
        ImgAnimationTomatoName);
  FShowcase[2].StandOn(205,0);
  for i:=0 to 2 do
  begin
    FShowcase[i].FrameRate:=Random(5-i)+i+1;
    FShowcase[i].Stop;
  end;
end;

destructor TAcrobaticsOne.Destroy;
begin
  FWallcloth.free;
  FPrincipal.free;
  FShowcase[0].free;
  FShowcase[1].free;
  FShowcase[2].free;
  inherited;
end;

procedure TAcrobaticsOne.Go;
begin
  inherited;
end;

procedure TAcrobaticsOne.Stay;
begin
  inherited Stay;
end;

{
*********************************** TPrecise ***********************************
}
constructor TPrecise.Create(AParent:TSprite);
begin
  inherited;
  FTickCount:=0;
end;

procedure TPrecise.SetEnabled(const Value: Boolean);
begin
  if Value then
  begin
    Visible:=True;
    Run;
  end
  else
  begin
    Visible:=False;
    Stop;
  end;
end;

{
*********************************** TExplode ***********************************
}
constructor TExplode.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreEffect.Items.Find(ImgEffectExplodeName);
  Width:=Image.Width;
  Height:=Image.Height;
  AnimCount:=2;
  AnimPos:=0;
  //  AnimSpeed:=1;
  //  AnimLooped:=True;
  FrameRate:=1;
  Enabled:=False;
end;

procedure TExplode.DoTimerMove(Sender:TObject);
begin
  inherited;
  Inc(FTickCount);
  if FTickCount>1 then
  begin
    Enabled:=False;
    FTickCount:=0;
  end;
  AnimPos:=AnimPos+1;
  if AnimPos>=AnimCount then AnimPos:=0;
end;

procedure TExplode.SetRole(const Value:Integer);
begin
  case Value of
    0: begin X:=270;y:=110; end;
    1: ;
  end;
end;

{
************************************* TOk **************************************
}
constructor TOk.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreContext.Items.Find(ImgContextOkName);
  Width:=Image.Width;
  Height:=Image.Height;
  Z:=5;
  Enabled:=False;
end;

procedure TOk.DoTimerMove(Sender:TObject);
begin
  inherited;
  Inc(FTickCount);
  if FTickCount>1 then
  begin
    Enabled:=False;
    FTickCount:=0;
  end;
end;

procedure TOk.SetRole(const Value:Integer);
begin
  case Value of
    0: begin X:=277;y:=130; end;
    1: ;
  end;
end;

{
********************************** TExtrinsic **********************************
}
procedure TExtrinsic.StandOn(vX,vY:Double);
begin
  X:=vX;
  Y:=vY;
end;

{
********************************** TChainBall **********************************
}
constructor TChainBall.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreBalls.Items.Find(ImgBallUnbrokenName);
  Width:=Image.Width;
  Height:=Image.Height;
end;

{
********************************** TBlueBall ***********************************
}
constructor TBlueBall.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreBalls.Items.Find(ImgBallBlueName);
  Width:=Image.Width;
  Height:=Image.Height;
end;

{
*********************************** TRedBall ***********************************
}
constructor TRedBall.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreBalls.Items.Find(ImgBallRedName);
  Width:=Image.Width;
  Height:=Image.Height;
end;

{
************************************ TFeces ************************************
}
constructor TFeces.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreBalls.Items.Find(ImgBallStoolName);
  Width:=Image.Width;
  Height:=Image.Height;
end;

{
************************************ TSmog *************************************
}
constructor TSmog.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreBalls.Items.Find(ImgBallNinjaName);
  Width:=Image.Width;
  Height:=Image.Height;
end;

{
*********************************** TGesture ***********************************
}
constructor TGesture.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreGesture.Items.Find(ImgGestureIdleName);
  Width:=Image.Width;
  Height:=Image.Height;
end;

procedure TGesture.SetValue(Value: Integer);
begin
  Image:=ImgStoreGesture.Items.Find(ImgGestureGestureName);
  Width:=Image.Width;
  Height:=Image.Height;
  Value:=Value-1;
  AnimPos:=Value div 2;
end;

{
******************************** TLiningGesture ********************************
}
constructor TLiningGesture.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreFixed.Items.Find('gesture');
  Width:=Image.Width;
  Height:=Image.Height;
  AnimCount:=3;
  AnimPos:=0;
end;

procedure TLiningGesture.SetStaus(const Value: TLiningType);
begin
  FStatus:=Value;
  AnimPos:=Ord(FStatus);
end;

procedure TLiningGesture.SetValue(Value: Integer);
begin
  Value:=Value-1;
  if Value=-1 then
  begin
    Image:=ImgStoreFixed.Items.Find('gray');
    Width:=Image.Width;
    Height:=Image.Height;
    AnimCount:=0;
    AnimPos:=0;
  end
  else
  begin
    Image:=ImgStoreFixed.Items.Find('gesture');
    Width:=Image.Width;
    Height:=Image.Height;
    AnimCount:=3;
    AnimPos:=0;
  end;
  case Value div 2 of
   0:
    Status:=ltRed;
   1:
    Status:=ltYellow;
   2:
    Status:=ltBlue;
  end;
end;

{
********************************** TScoreNum ***********************************
}
constructor TScoreNum.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreContext.Items.Find(ImgContextScoreName);
  Width:=Image.Width;
  Height:=Image.Height;
  AnimCount:=10;
end;

{
*********************************** TMarker ************************************
}
constructor TMarker.Create;
var
  i: Integer;
begin
  inherited Create;
  
  for i:=0 to Length(FDigits)-1 do
  begin
    FDigits[i]:=TScoreNum.Create(LayoutThree.Engine);
    FDigits[i].Visible:=False;
  end;
  FValue:=0;
  FNoteCount:=0;
end;

procedure TMarker.Display;
var
  d: Integer;
begin
  d:=FValue mod 10;
  FDigits[0].AnimPos:=d;
  d:=(FValue div 10) mod 10;
  FDigits[1].AnimPos:=d;
  d:=(FValue div 100) mod 10;
  FDigits[2].AnimPos:=d;
  d:=(FValue div 1000) mod 10;
  FDigits[3].AnimPos:=d;
  d:=(FValue div 10000) mod 10;
  FDigits[4].AnimPos:=d;
end;

procedure TMarker.Inc(Value:Integer);
begin
  FValue:=FValue+Value;
  display;
end;

procedure TMarker.IncNote;
begin
  FNoteCount:=FNoteCount+1;
end;

procedure TMarker.Reset;
begin
  FValue:=0;
  Display;
end;

procedure TMarker.SetRole(const Value: Integer);
var
  i: Integer;
begin
  for i:=0 to Length(FDigits)-1 do
    case Value of
      0:
      begin
        FDigits[i].StandOn(290-i*10,7);
        FDigits[i].Visible:=True;
      end;
      1: ;
    end;
end;

{
********************************** TComboNum ***********************************
}
constructor TComboNum.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreContext.Items.Find(ImgContextScoreName);
  Width:=Image.Width;
  Height:=Image.Height;
  AnimCount:=10;
  AnimPos:=0;
end;

procedure TComboNum.SetDigit(const Value: Integer);
begin
  FDigit := Value;
  AnimPos:=FDigit;
end;

{
*********************************** TRule1P ************************************
}
constructor TRule1P.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreFixed.Items.Find('Rule1P');
  Width:=Image.Width;
  Height:=Image.Height;
  X:=275;
  Y:=117;
end;

{
************************************ TCombo ************************************
}
constructor TCombo.Create;
begin
  FTitle:=TComboImg.Create(LayoutSix.Engine);
  FTitle.Visible:=False;
  //FTitle.X:=380;
  //FTitle.Y:=120;
  FValue:=TComboValue.Create;
  FValue.Visible:=False;
end;

destructor TCombo.Destroy;
begin
  inherited;
  
end;

procedure TCombo.Clear;
begin
  FValue.Value:=0;
  FTitle.Visible:=False;
end;

function TCombo.GetValue: Integer;
begin
  Result:=FValue.Value;
end;

procedure TCombo.Inc;
begin
  SetValue(Value+1);
end;

procedure TCombo.SetRole(const Value: Integer);
begin
  case Value of
    0:
    begin
      FTitle.StandOn(380,120);
      FValue.StandOn(390,130);
  //      FTitle.Visible:=True;
        //FValue: TComboValue;
    end;
    1: ;
  end;
end;

procedure TCombo.SetValue(const Value: Integer);
begin
  FValue.Value:=Value;
  FTitle.Visible:=False;
  FValue.Visible:=False;
  if Value>0 then
  begin
    FTitle.Visible:=True;
    FValue.Visible:=True;
  end;
end;

{
********************************* TPercentNum **********************************
}
constructor TPercentNum.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreContext.Items.Find(ImgContextQualified1Name);
  Width:=Image.Width;
  Height:=Image.Height;
  AnimCount:=10;
  AnimPos:=0;
end;

procedure TPercentNum.SetDigit(const Value: Integer);
begin
  FDigit := Value;
  AnimPos:=FDigit;
end;

{
********************************* TPercentSign *********************************
}
constructor TPercentSign.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreContext.Items.Find(ImgContextPercentSignName);
  Width:=Image.Width;
  Height:=Image.Height;
end;

{
********************************** TPercentag **********************************
}
constructor TPercentag.Create;
begin
  inherited Create;
  
  FDigitTen:=TPercentNum.Create(LayoutThree.Engine);
  FDigitTen.Visible:=False;
  FDigitUnit:=TPercentNum.Create(LayoutThree.Engine);
  FDigitUnit.Visible:=False;
  FSign:=TPercentSign.Create(LayoutThree.Engine);
  FSign.Visible:=False;
end;

procedure TPercentag.SetRole(const Value: Integer);
begin
  case Value of
    0:
    begin
      FDigitTen.StandOn(305,30);
      FDigitUnit.StandOn(340,30);
      FSign.StandOn(370,30);
      FDigitTen.Visible:=True;
      FDigitUnit.Visible:=True;
      FSign.Visible:=True;
    end;
    1: ;
  end;
end;

procedure TPercentag.SetValue(const Value: Integer);
var
  d: Integer;
begin
  d:=Value mod 10;
  FDigitUnit.SetDigit(d);
  d:=(Value-d) div 10;
  if d>9 then d:=9;
  if d<0 then d:=0;
  FDigitTen.SetDigit(d);
end;

{
********************************** TComboImg ***********************************
}
constructor TComboImg.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreContext.Items.Find(ImgContextComboName);
  Width:=Image.Width;
  Height:=Image.Height;
end;

{
********************************* TBackScenery *********************************
}
constructor TBackScenery.Create(AParent:TSprite);
begin
  inherited;
  X:=0;
  Y:=0;
end;

{
********************************* TYellowBall **********************************
}
constructor TYellowBall.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreBalls.Items.Find(ImgBallYellowName);
  Width:=Image.Width;
  Height:=Image.Height;
end;

{
********************************** TSongName ***********************************
}
constructor TSongName.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreContext.Items.Find(ImgContextSongName);
  Width:=Image.Width;
  Height:=Image.Height;
  X:=45;
  Y:=7;
end;

{
******************************* TBackPipeLineTwo *******************************
}
constructor TBackPipeLineTwo.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreFixed.Items.Find(ImgPlayerTwoPipeBackName);
  Width:=Image.Width;
  Height:=Image.Height;
  AnimCount:=2;
  AnimPos:=0;
end;

{
********************************* TComboValue **********************************
}
constructor TComboValue.Create;
begin
  FDigitTen:=TComboNum.Create(LayoutSix.Engine);
  FDigitTen.Visible:=False;
  FDigitUnit:=TComboNum.Create(LayoutSix.Engine);
  FDigitUnit.Visible:=False;
end;

function TComboValue.GetValue: Integer;
var
  v: Integer;
begin
  v:=Round(FDigitTen.animPos*10+FDigitUnit.animPos);
  Result:=v;
end;

procedure TComboValue.SetValue(Value: Integer);
var
  d: Integer;
begin
  if Value>99 then Value:=00;
  if Value<0 then Value:=0;
  d:=Value mod 10;
  FDigitUnit.SetDigit(d);
  d:=(Value - d) div 10;
  FDigitTen.SetDigit(d);
end;

procedure TComboValue.SetVisible(const Value: Boolean);
begin
  FDigitTen.Visible:=Value;
  FDigitUnit.Visible:=Value;
end;

procedure TComboValue.StandOn(vX,vY:Integer);
begin
  FDigitTen.StandOn(vX,vY);
  FDigitUnit.StandOn(vX+10,vY);
end;

{
********************************** TStepMark ***********************************
}
constructor TStepMark.Create(AParent:TSprite;ImgIndex:Integer);
begin
  inherited Create(AParent);
  Image:=ImgStoreFixed.Items.Find(ImgFixedStratumName);
  Width:=Image.Width;
  Height:=Image.Height;
  AnimPos:=ImgIndex;
end;

{
********************************* TStressValue *********************************
}
constructor TStressValue.Create;
begin
  FCount:=0;
  FHidDev:=nil;
  FLastValue:=0;
  FNeedRelease:=False;;
  FRelease:=False;
  FValue:=0;
  FEasyMode:=False;
end;

procedure TStressValue.Refresh;
var
  buf: array[0..8] of Byte;
  toread: Cardinal;
  read: Cardinal;
  v: Integer;
begin
  buf[0]:=0;buf[1]:=0;
  if assigned(FHidDev) then
  begin
    toread:=FHidDev.Caps.InputReportByteLength;
    HidCheck(FHidDev.ReadFile(buf,toread,read));
    v:=buf[1] div 5;
    if FEasyMode then
    begin
      Value:=v;
      exit;
    end;
    if FRelease then
      Value:=v
    else
      Value:=0;
    if (v=0) or (v=1) then
    begin
       FRelease:=True;
    end;
    if (FLastValue>v) then
      FRelease:=False;
    if FLastValue=v then
    begin
      Inc(FCount);
      if FCount>35 then FRelease:=False;
    end else FCount:=0;
    FLastValue:=v;
  end;
end;

{
******************************* TStressValueOne ********************************
}
constructor TStressValueOne.Create;
var
  i: Integer;
begin
  for i:=0 to 5 do
  begin
    FStepMarks[i]:=TStepMark.Create(LayoutFive.Engine,i);
    FStepMarks[i].StandOn(344,162-i*10);
  end;
  FStepMarks[0].StandOn(346,Round(FStepMarks[0].Y));
  FStepMarks[5].StandOn(347,Round(FStepMarks[5].Y));
  FStepMarks[1].StandOn(345,Round(FStepMarks[1].Y));
  FStepMarks[4].StandOn(345,Round(FStepMarks[4].Y));
end;

destructor TStressValueOne.Destroy;
var
  i: Integer;
begin
  for i:=0 to 5 do
    FStepMarks[i].Free;
  inherited;
end;

procedure TStressValueOne.SetValue(Value:Integer);
var
  i: Integer;
begin
  if Value>6 then Value:=6;
  FValue:=Value;
  for i:=0 to 5 do
    FStepMarks[i].Visible:=False;
  for i:=1 to FValue do
    FStepMarks[i-1].Visible:=True;
end;

{
******************************* TBackSceneryOne ********************************
}
constructor TBackSceneryOne.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreFixed.Items.Find('Back1p');
  Width:=Image.Width;
  Height:=Image.Height;
  x:=33;
  y:=5;
  AnimCount:=2;
  AnimPos:=0;
  FrameRate:=4;
end;

procedure TBackSceneryOne.DoTimerMove(Sender:TObject);
begin
  inherited;
  AnimPos:=AnimPos+1;
  if AnimPos>=AnimCount then AnimPos:=0;
end;

{ TGripGame }

{
*********************************** TMeter1P ***********************************
}
constructor TMeter1P.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreFixed.Items.Find(ImgPlayerOneMeterName);
  Width:=Image.Width;
  Height:=Image.Height;
  X:=340;
  Y:=83;
end;

{
*********************************** TMeter2P ***********************************
}
constructor TMeter2P.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreFixed.Items.Find(ImgPlayerTwoMeterName);
  Width:=Image.Width;
  Height:=Image.Height;
  X:=341;
  Y:=255;
end;

{
********************************** TToyArise ***********************************
}
constructor TToyArise.Create;
begin
  FGesture:=TGesture.Create(LayoutFive.Engine);
  FGesture.Visible:=False;
  FLiningGesture:=TLiningGesture.Create(LayoutFour.Engine);
  FLiningGesture.Visible:=False;
end;

destructor TToyArise.Destroy;
begin
  FGesture.Free;
  FLiningGesture.free;
  inherited;
end;

procedure TToyArise.SetRole(const Value: Integer);
begin
  case Value of
    0:
    begin
      FMeter:=TMeter1P.Create(LayoutFour.Engine);
      FLiningGesture.StandOn(GesturePositionOne.x,
            GesturePositionOne.y);
      FGesture.StandOn(332,69);
      FStressValue:=TStressValueOne.Create;
      FGesture.Visible:=True;
      FLiningGesture.Visible:=True;
    end;
    1:
    begin
      FMeter:=TMeter2P.Create(LayoutFour.Engine);
      FLiningGesture.StandOn(GesturePositionOne.x,
            GesturePositionOne.y);
      FGesture.StandOn(332,239);
      FStressValue:=TStressValueTwo.Create;
      FGesture.Visible:=True;
      FLiningGesture.Visible:=True;
    end;
  end;
end;

procedure TToyArise.SetValue(const Value: Integer);
begin
  FGesture.Value:=Value;
  FLiningGesture.Value:=Value;
end;

{
********************************** TPipeLine ***********************************
}
constructor TPipeLine.Create;
begin
  FRuleMark:=TRuleMark.Create(LayoutThree.Engine);
  FRuleMark.Visible:=False;
end;

destructor TPipeLine.Destroy;
begin
  FRuleMark.free;
  inherited;
end;

function TPipeLine.CreatePrize(PrizeKind:TPrizeKind): TPrize;
var
  Note: TPrize;
begin
  case PrizeKind of
    pkBlue:
    begin
      Note:=TBlueBall.Create(LayoutThree.Engine);
  //      Note.FrameRate:=3;
    end;
    pkYellow:
    begin
      Note:=TYellowBall.Create(LayoutThree.Engine);
  //      Note.FrameRate:=3;
    end;
    pkRed:
    begin
      Note:=TRedBall.Create(LayoutThree.Engine);
  //      Note.FrameRate:=3;
    end;
    pkChain:
    begin
      Note:=TChainBall.Create(LayoutThree.Engine);
  //      Note.FrameRate:=3;
    end;
    pkLoo:
    begin
      Note:=TFeces.Create(LayoutThree.Engine);
  //      Note.FrameRate:=3;
    end;
    pkHammer:
    begin
      Note:=TBlueBall.Create(LayoutThree.Engine);
  //      Note.FrameRate:=3;
    end;
    pkSmog:
    begin
      Note:=TBlueBall.Create(LayoutThree.Engine);
  //      Note.FrameRate:=3;
    end;
  end;
  Note.Z:=10;
  Result:=Note;
end;

procedure TPipeLine.NewPrize(PrizeKind:TPrizeKind);
begin
end;

procedure TPipeLine.Shift(Steps:Integer);
var
  i: Integer;
  x: Double;
begin
  for i:=0 to Length(FNotes)-1 do
  begin
    if assigned(FNotes[i]) then
    begin
      FNotes[i].Move(Steps);
     // x:=FNotes[i].X;
     // x:=X+1*Steps;
     // FNotes[i].X:=x;
    end;
  end;
end;

procedure TPipeLine.StepOne;
var
  i: Integer;
  x: Double;
begin
  for i:=0 to Length(FNotes)-1 do
  begin
    x:=FNotes[i].X;
    x:=X+10;
    FNotes[i].X:=x;
  end;
end;

procedure TPipeLine.TurnLive(Index:Integer);
begin
  {if Index>Length(FNotes) then exit;
  FNotes[Index].StandOn(0,Round(FNotes[Index].Y));
  FNotes[Index].Run;}
end;

{
********************************* TPipeLineOne *********************************
}
constructor TPipeLineOne.Create;
begin
  inherited Create;

  FRule:=TRule1P.Create(LayoutThree.Engine);
  FBackPipeLine:=TBackPipeLineOne.Create(LayoutTwo.Engine);
  FBackPipeLine.Visible:=False;
  FBackPipeLine.StandOn(204,112);
  FRuleMark.Role:=0;
end;

destructor TPipeLineOne.Destroy;
begin
  FRule.free;
  FBackPipeLine.free;
  inherited;
end;

procedure TPipeLineOne.Go;
var
  i: Integer;
begin
  //    for i:=0 to Length(FNotes)-1 do
  //      FNotes[i].Run;
end;

procedure TPipeLineOne.Move(Step:Integer);
var
  i: Integer;
  j: Integer;
begin
  for i:=0 to Length(FNotes)-1 do
    for j:=0 to Step do
      FNotes[i].DoTimerMove(Self);
end;

{
******************************** TSunshadeRight ********************************
}
constructor TSunshadeRight.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreFixed.Items.Find('sunshaderight');
  Width:=Image.Width;
  Height:=Image.Height;
  X:=433;
  Y:=87;
end;

{
******************************** TQualifiedGoal ********************************
}
constructor TQualifiedGoal.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreContext.Items.Find(ImgContextQualifiedRateName);
  Width:=Image.Width;
  Height:=Image.Height;
  X:=320;
  Y:=7;
end;

{
******************************** TPlayerScreen *********************************
}
constructor TPlayerScreen.Create;
begin
  FToyArise:=TToyArise.Create;
  FCombo:=TCombo.Create;
  FPercentag:=TPercentag.Create;
  FMarker:=TMarker.Create;
end;

destructor TPlayerScreen.Destroy;
begin
  FToyArise.Free;
  FCombo.Free;
  FPercentag.Free;
  FMarker.Free;
  FPipeLine.Free;
  inherited;
end;

procedure TPlayerScreen.Go;
begin
  FAcrobatics.Go;
end;

procedure TPlayerScreen.NewBall;
begin
  FPipeLine.NewPrize(pkBlue);
end;

procedure TPlayerScreen.Stay;
begin
  FAcrobatics.Stay;
end;

{ TPlayerScreen }

{
******************************* TStressValueTwo ********************************
}
constructor TStressValueTwo.Create;
var
  i: Integer;
begin
  for i:=0 to 5 do
  begin
    FStepMarks[i]:=TStepMark.Create(LayoutFour.Engine,i);
    FStepMarks[i].StandOn(340,330-i*10);
  end;
{  FStepMarks[0].StandOn(346,Round(FStepMarks[0].Y));
  FStepMarks[5].StandOn(347,Round(FStepMarks[5].Y));
  FStepMarks[1].StandOn(345,Round(FStepMarks[1].Y));
  FStepMarks[4].StandOn(345,Round(FStepMarks[4].Y));}
end;

destructor TStressValueTwo.Destroy;
var
  i: Integer;
begin
  for i:=0 to 5 do
    FStepMarks[i].Free;
  inherited;
end;

{ TToyArise }

{ TCombo }

{ TPrecise }

{ TScoreNum }

{
*********************************** THammer ************************************
}
constructor THammer.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreBalls.Items.Find(ImgBallHammerName);
  Width:=Image.Width;
  Height:=Image.Height;
end;

{ TSunshadeRight }

{ TSunshadeLeft }

{
******************************* TPlayerOneScreen *******************************
}
constructor TPlayerOneScreen.Create;
begin
  inherited Create;

  FPipeLine:=TPipeLineOne.Create;
  FAcrobatics:=TAcrobaticsOne.Create;
  FToyArise.Role:=0;
  FCombo.Role:=0;
  FPercentag.Role:=0;
  FMarker.Role:=0;
  FSunshadeLeft:=TSunshadeLeft.Create(LayoutSix.Engine);
  FSunshadeRight:=TSunshadeRight.Create(LayoutSix.Engine);
  FSongName:=TSongName.Create(LayoutFive.Engine);
  FQualifiedGoal:=TQualifiedGoal.Create(LayoutFive.Engine);
end;

destructor TPlayerOneScreen.Destroy;
begin
  FPipeLine.Free;
  FAcrobatics.Free;
  FSunshadeLeft.Free;
  FSunshadeRight.Free;
  FSongName.Free;
  FQualifiedGoal.Free;
  inherited;
end;

{
********************************** TGripGame ***********************************
}
constructor TGripGame.Create;
begin
  FPlayer1:=TPlayerOneScreen.Create;
  FPlayer2:=TPlayerTwoScreen.Create;
  FRocker:=TRocker.Create;
  FMetronome:=TMetronome.Create;
end;

destructor TGripGame.Destroy;
begin
  FPlayer1.Free;
  FPlayer2.Free;
  FRocker.Free;
  FMetronome.Free;
  inherited;
end;

procedure TGripGame.ForTest;
begin
    //  FPlayer1.NewBall;
  FMetronome.ReadPrizeList('Fine.txt');
  FPlayer1.FPipeLine.Spread:=FMetronome.Spread;
  FRocker.Play;
  //  FPlayer1.FPipeLine.Go;
end;

function TGripGame.GetComboOne: TCombo;
begin
  Result:=Self.FPlayer1.FCombo;
end;

function TGripGame.GetMakerOne: TMarker;
begin
  Result:=FPlayer1.FMarker;
end;

function TGripGame.GetPercentagOne: TPercentag;
begin
  Result:=FPlayer1.FPercentag;
end;

function TGripGame.GetStressOne: TStressValue;
begin
  Result:=FPlayer1.FToyArise.FStressValue;
end;

function TGripGame.GetStressTwo: TStressValue;
begin
  Result:=FPlayer2.FToyArise.FStressValue;
end;

procedure TGripGame.NotifyDevice;
begin
  StressOne.Refresh;
  FPlayer1.FToyArise.Value:=StressOne.Value;
  if StressOne.Value>0 then
    FPlayer1.FPipeLine.FBackPipeLine.Visible:=True
  else
    FPlayer1.FPipeLine.FBackPipeLine.Visible:=False;
  ScreenForm.Edit1.Text:=IntToStr(StressOne.Value);
  //Datas.Memo1.Lines.Add(IntToStr(StressOne.Value));
end;

procedure TGripGame.NotifyNote(Note:TNoteRec);
  
  const
    lastidx:Integer=0;
  var
    mesure:integer;
  
begin
  mesure:=(Note.bar*4+Note.beat)*100+Note.tick;
  if (mesure<>lastidx)then
  begin
   FPlayer1.FPipeLine.Shift(mesure-lastidx);
   FPlayer2.FPipeLine.Shift(mesure-lastidx); 
   lastidx:=mesure;
   {Datas.Memo1.Lines.Add(IntToStr(Note.bar)+':'+
                         IntToStr(NOte.beat)+':'+
                         IntToStr(NOte.tick));}
  end;
end;

procedure TGripGame.Restart;
begin
  FPlayer1.Go;
  FPlayer2.Go;
  FMetronome.ReadPrizeList('Fine.txt');
  FPlayer1.FPipeLine.Spread:=FMetronome.Spread;
  FPlayer2.FPipeLine.Spread:=FMetronome.Spread;
  FRocker.Play;
end;

procedure TGripGame.Resume;
begin
  //  FPlayer1.Stay;
  FRocker.Play;
end;

procedure TGripGame.Stop;
begin
  FPlayer1.Stay;
  FPlayer2.Stay;
  FRocker.Stop;
end;

{ TStressVale }

{
******************************* TBackPipeLineOne *******************************
}
constructor TBackPipeLineOne.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreFixed.Items.Find(ImgPlayerOnePipeBackName);
  Width:=Image.Width;
  Height:=Image.Height;
end;

{ TQualifiedGoal }

{
******************************** TSunshadeLeft *********************************
}
constructor TSunshadeLeft.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreFixed.Items.Find('sunshadeleft');
  Width:=Image.Width;
  Height:=Image.Height;
  X:=1;
  Y:=106;
end;

{ TComboValue }

{
*********************************** TRule2P ************************************
}
constructor TRule2P.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreFixed.Items.Find('Rule2P');
  Width:=Image.Width;
  Height:=Image.Height;
  X:=275;
  Y:=290;
end;

{ TPrize }

{
*********************************** TActive ************************************
}
constructor TActive.Create(AParent:TSprite);
begin
  inherited;
  FTimer:=TTimer.Create(Application);
  FTimer.OnTimer:=DoTimerMove;
  FTimer.Enabled:=False;
  FTimer.Interval:=100;
end;

destructor TActive.Destroy;
begin
  inherited;
  FTimer.Free;
end;

procedure TActive.DoTimerMove(Sender:TObject);
begin
  Move(1);
end;

function TActive.GetFrameRate: Integer;
begin
  Result:=1000 div FTimer.Interval;
end;

procedure TActive.Run;
begin
  FTimer.Enabled:=True;
end;

procedure TActive.SetFrameRate(const Value: Integer);
begin
  FTimer.Interval:=Value*100;
  FTimer.Enabled:=True;
end;

procedure TActive.Stop;
begin
  FTimer.Enabled:=False;
end;

{ TRuleMark }

{
********************************** TRuleMark ***********************************
}
constructor TRuleMark.Create(AParent:TSprite);
begin
  inherited;
  Image:=ImgStoreFixed.Items.Find('detect');
  Width:=Image.Width;
  Height:=Image.Height;
  z:=5;
  AnimCount:=1;
  AnimPos:=0;
  AnimSpeed:=1;
  AnimLooped:=True;
end;

procedure TRuleMark.DoCollision(Sprite:TSprite;var Done:Boolean);
var
  d: Double;
  combo: Boolean;
begin
  inherited;
  if (Sprite is TPrize) and (Sprite.Visible) and (
    (Sprite.X>X-10) and (Sprite.X<X+10)) then
  begin
    combo:=False;
    TPrize(Sprite).Overline;
    if (StressOne.Value>0) and (Sprite is TBlueBall) then
    begin
      TPrize(Sprite).ToDead;
      Explode.Role:=0;
      Explode.Enabled:=True;
      MarkerOne.Inc(50);
      MarkerOne.IncNote;
      Ok.Role:=0;
      Ok.Enabled:=True;
      ComboOne.Inc;
      combo:=True;
      PlaySound(PChar(AppPath+'good.wav'),0,SND_FILENAME or SND_NOWAIT or SND_ASYNC);
    end;
    if ((StressOne.Value=1) or (StressOne.Value=2)) and
      (Sprite is TYellowBall) then
    begin
      TPrize(Sprite).ToDead;
      Explode.Role:=0;
      Explode.Enabled:=True;
      MarkerOne.Inc(50);
      MarkerOne.IncNote;
      Ok.Role:=0;
      Ok.Enabled:=True;
      ComboOne.Inc;
      combo:=True;
      PlaySound(PChar(AppPath+'good.wav'),0,SND_FILENAME or SND_NOWAIT or SND_ASYNC);
    end;
    if ((StressOne.Value=5) or (StressOne.Value=6)) and
      (Sprite is TRedBall) then
    begin
      TPrize(Sprite).ToDead;
      Explode.Role:=0;
      Explode.Enabled:=True;
      MarkerOne.Inc(50);
      MarkerOne.IncNote;
      Ok.Role:=0;
      Ok.Enabled:=True;
      ComboOne.Inc;
      combo:=True;
      PlaySound(PChar(AppPath+'good.wav'),0,SND_FILENAME or SND_NOWAIT or SND_ASYNC);
    end;
    if TPrize(Sprite).Tag<100 then
      d:=(MarkerOne.NoteCount / 100)*100
    else
      d:=(MarkerOne.NoteCount / TPrize(Sprite).Tag)*100;
    PercentagOne.Value:=Round(d);
    Datas.Memo1.Lines.Add('Value :'+IntToStr(Round(d)));
    if not combo then ComboOne.Clear;
  end;
end;

procedure TRuleMark.DoMove(MoveCount:Integer);
begin
  inherited;
  Collision;
end;

procedure TRuleMark.SetRole(const Value: Integer);
begin
  inherited;
  case Value of
    0: begin   X:=275; Y:=117;  end;
    1: begin   X:=275; Y:=290;  end;
  end;
  Visible:=True;
end;

{ TPrize }

{
************************************ TPrize ************************************
}
constructor TPrize.Create(AParent:TSprite);
begin
  inherited;
  FTag:=0;
  FDead:=False;
  FDeadMovePixel:=0;
  FOverline:=False;
end;

procedure TPrize.Move(Step:Integer);
var
  _x: Double;
begin
  if FDead then
  begin
    Inc(FDeadMovePixel);
    if FDeadMovePixel>10 then
    begin
      Visible:=False;
    end;
  end;
  if FOverline then
  begin
    Inc(FDeadMovePixel);
    if FDeadMovePixel>15 then
    begin
  //      PlaySound(PChar(AppPath+'miss.wav'),0,SND_FILENAME or SND_NOWAIT or SND_ASYNC);
    end;
  end;
  _x:=X;
  if ScreenForm.CheckBox3.Checked then
    _x:=X+0.5*Step
  else
    _x:=X+1*Step;
  X:=_x;
end;

procedure TPrize.Overline;
begin
  FOverline:=True;
end;

procedure TPrize.ToDead;
begin
  FDead:=True;
end;

{ TPlayerTwoScreen }

constructor TPlayerTwoScreen.Create;
begin
  inherited;
  FPipeLine:=TPipeLineTwo.Create;
  FAcrobatics:=TAcrobaticsTwo.Create;
  FToyArise.Role:=1;
  FCombo.Role:=1;
  FPercentag.Role:=1;
  FMarker.Role:=1;
  FSunshadeLeft:=TSunshadeLeft.Create(LayoutSix.Engine);
  FSunshadeRight:=TSunshadeRight.Create(LayoutSix.Engine);
end;

destructor TPlayerTwoScreen.Destroy;
begin

  inherited;
end;

{ TPipeLineTwo }

constructor TPipeLineTwo.Create;
begin
  inherited;
  FRule:=TRule2P.Create(LayoutThree.Engine);
  FBackPipeLine:=TBackPipeLineTwo.Create(LayoutTwo.Engine);
  FBackPipeLine.Visible:=False;
  FBackPipeLine.StandOn(204,212);
  FRuleMark.Role:=1;
end;

destructor TPipeLineTwo.Destroy;
begin

  inherited;
end;

procedure TPipeLineTwo.Go;
begin
  inherited;

end;

procedure TPipeLineTwo.Move(Step: Integer);
begin

end;

procedure TPipeLineTwo.SetBringBalls(Spread: TBits);
var
  i: Integer;
  c: Integer;
  x: Double;
  k: Integer;
  label ex;

begin
    c:=-1;
    for i:=0 to Spread.Size-1 do
    begin
      if Spread.Bits[i] then
        Inc(c);
    end;
    if Length(FNotes)>0 then
    begin
      for i:=0 to Length(FNotes)-1 do
        FNotes[i].Free;
    end;
    SetLength(FNotes,c);
    c:=0;
    x:=775;
    i:=0;
    while i<(Spread.Size-1) do
    begin
      if Spread.Bits[i] then
      begin
        k:=Random(6);
        if k>0 then
        begin
          k:=Random(6);
          if k=1 then k:=Random(6);
        end;
        if k>1 then
        begin
         k:=Random(6);
         if k=2 then k:=Random(6);
        end;
        if k>2 then
        begin
         k:=Random(6);
         if k=3 then k:=Random(6);
        end;
        if not (ScreenForm.CheckBox2.Checked) then
        begin
          if (c=72) or (c=23) or (c=29) or (c=35) or
             (c=51) then
          begin
            FNotes[c]:=CreatePrize(pkChain);
            FNotes[c].StandOn(x,292);
            goto ex;
          end;
        end;
        if k=0 then
        begin
          FNotes[c]:=CreatePrize(pkBlue);
          FNotes[c].StandOn(x,292);
        end;
        if k=1 then
        begin
          if not (ScreenForm.CheckBox2.Checked) then
          begin
            FNotes[c]:=CreatePrize(pkRed);
            FNotes[c].StandOn(x,282);
          end
          else
          begin
            FNotes[c]:=CreatePrize(pkBlue);
            FNotes[c].StandOn(x,292);
          end;
        end;
        if k=2 then
        begin
          if not (ScreenForm.CheckBox2.Checked) then
          begin
            FNotes[c]:=CreatePrize(pkYellow);
            FNotes[c].StandOn(x,298);
          end
          else
          begin
            FNotes[c]:=CreatePrize(pkBlue);
            FNotes[c].StandOn(x,292);
          end;
        end;
        if k=3 then
        begin
          FNotes[c]:=CreatePrize(pkLoo);
          FNotes[c].StandOn(x,292);
        end;
        if k=4 then
        begin
          FNotes[c]:=CreatePrize(pkHammer);
          FNotes[c].StandOn(x,292);
        end;
        if k=5 then
        begin
          FNotes[c]:=CreatePrize(pkSmog);
          FNotes[c].StandOn(x,292);
        end;
          //FNotes[c].FrameRate:=1;
  ex:
        FNOtes[c].Stop;
        FNotes[c].Tag:=c+1;
        Inc(c);
      end;
      if ScreenForm.CheckBox3.Checked then
        x:=x-0.5
      else
        x:=x-1;
      Inc(i);
    end;
    {  for i:=0 to Spread.Size-1 do
      begin
        if Spread.Bits[i] then
        begin
          k:=Random(3);
          if k=0 then
          begin
            FNotes[c]:=CreatePrize(pkBlue);
            FNotes[c].StandOn(x,122);
          end;
          if k=1 then
          begin
            FNotes[c]:=CreatePrize(pkRed);
            FNotes[c].StandOn(x,112);
          end;
          if k=2 then
          begin
            FNotes[c]:=CreatePrize(pkYellow);
          FNotes[c].StandOn(x,128);
          end;
          //FNotes[c].FrameRate:=1;
          FNOtes[c].Stop;
          Inc(c);
        end;
        Dec(x,1);
      end;}
end;

{ TAcrobaticsTwo }

constructor TAcrobaticsTwo.Create;
var
  i: Integer;
begin
  inherited Create;

  FWallcloth:=TBackSceneryTwo.Create(LayoutTwo.Engine);
  FPrincipal:=TPrincipal.Create(LayoutThree.Engine,
                  ImgAnimationJumpPlayerTwoName);
  FPrincipal.FrameRate:=4;
  FPrincipal.StandOn(30,168);
  FPrincipal.Stop;
  FShowcase[0]:=TWebbitToy.Create(LayoutThree.Engine,
        ImgAnimationCubeName);
  FShowcase[0].StandOn(115,170);
  FShowcase[1]:=TWebbitToy.Create(LayoutThree.Engine,
        ImgAnimationCylinderName);
  FShowcase[1].StandOn(135,155);
  FShowcase[2]:=TWebbitToy.Create(LayoutThree.Engine,
        ImgAnimationTomatoName);
  FShowcase[2].StandOn(205,170);
  for i:=0 to 2 do
  begin
    FShowcase[i].FrameRate:=Random(5-i)+i+1;
    FShowcase[i].Stop;
  end;
end;

destructor TAcrobaticsTwo.Destroy;
begin

  inherited;
end;

procedure TAcrobaticsTwo.Go;
begin
  inherited;

end;

procedure TAcrobaticsTwo.Stay;
begin
  inherited;

end;

procedure TPipeLineOne.SetBringBalls(Spread: TBits);
var
  i: Integer;
  c: Integer;
  x: Double;
  k: Integer;

  label ex;

begin
    c:=-1;
    for i:=0 to Spread.Size-1 do
    begin
      if Spread.Bits[i] then
        Inc(c);
    end;
    if Length(FNotes)>0 then
    begin
      for i:=0 to Length(FNotes)-1 do
        FNotes[i].Free;
    end;
    SetLength(FNotes,c);
    c:=0;
    x:=775;
    i:=0;
    while i<(Spread.Size-1) do
    begin
      if Spread.Bits[i] then
      begin
        k:=Random(6);
        if k>0 then
        begin
          k:=Random(6);
          if k=1 then k:=Random(6);
        end;
        if k>1 then
        begin
         k:=Random(6);
         if k=2 then k:=Random(6);
        end;
        if k>2 then
        begin
         k:=Random(6);
         if k=3 then k:=Random(6);
        end;
        if not (ScreenForm.CheckBox2.Checked) then
        begin
          if (c=72) or (c=23) or (c=29) or (c=35) or
             (c=51) then
          begin
            FNotes[c]:=CreatePrize(pkChain);
            FNotes[c].StandOn(x,122);
            goto ex;
          end;
        end;
        if k=0 then
        begin
          FNotes[c]:=CreatePrize(pkBlue);
          FNotes[c].StandOn(x,122);
        end;
        if k=1 then
        begin
          if not (ScreenForm.CheckBox2.Checked) then
          begin
            FNotes[c]:=CreatePrize(pkRed);
            FNotes[c].StandOn(x,112);
          end
          else
          begin
            FNotes[c]:=CreatePrize(pkBlue);
            FNotes[c].StandOn(x,122);
          end;
        end;
        if k=2 then
        begin
          if not (ScreenForm.CheckBox2.Checked) then
          begin
            FNotes[c]:=CreatePrize(pkYellow);
            FNotes[c].StandOn(x,128);
          end
          else
          begin
            FNotes[c]:=CreatePrize(pkBlue);
            FNotes[c].StandOn(x,122);
          end;
        end;
        if k=3 then
        begin
          FNotes[c]:=CreatePrize(pkLoo);
          FNotes[c].StandOn(x,122);
        end;
        if k=4 then
        begin
          FNotes[c]:=CreatePrize(pkHammer);
          FNotes[c].StandOn(x,122);
        end;
        if k=5 then
        begin
          FNotes[c]:=CreatePrize(pkSmog);
          FNotes[c].StandOn(x,122);
        end;
          //FNotes[c].FrameRate:=1;
  ex:
        FNOtes[c].Stop;
        FNotes[c].Tag:=c+1;
        Inc(c);
      end;
      if ScreenForm.CheckBox3.Checked then
        x:=x-0.5
      else
        x:=x-1;
      Inc(i);
    end;
    {  for i:=0 to Spread.Size-1 do
      begin
        if Spread.Bits[i] then
        begin
          k:=Random(3);
          if k=0 then
          begin
            FNotes[c]:=CreatePrize(pkBlue);
            FNotes[c].StandOn(x,122);
          end;
          if k=1 then
          begin
            FNotes[c]:=CreatePrize(pkRed);
            FNotes[c].StandOn(x,112);
          end;
          if k=2 then
          begin
            FNotes[c]:=CreatePrize(pkYellow);
          FNotes[c].StandOn(x,128);
          end;
          //FNotes[c].FrameRate:=1;
          FNOtes[c].Stop;
          Inc(c);
        end;
        Dec(x,1);
      end;}
end;

end.
