unit uUtility;

interface

uses
  windows,classes,SysUtils,graphics,uConst,HidControllerClass;

  function CreateBitmapRgn(Bitmap: TBitmap; TransparentColor: TColor): HRgn;
  function XYInMiniIcon(X, Y: Integer): boolean;
  function XYInCloseIcon(X, Y: Integer): boolean;
  function GetDeviceToToy(Device:TJvHidDevice;ProductName:string;
          Pid:Integer;Vid:Integer):boolean;
  procedure SplitToBarPos(s:string;var bar,beat,tick,kind:Integer);

implementation

function CreateBitmapRgn(Bitmap: TBitmap; TransparentColor: TColor): HRgn;
var
  BitmapMask: TBitmap;
  RectList: TList;
  RgnData: PRgnData;
  I, Size: Integer;
  Rects: PRect;

  procedure FindRegionRectangles;
  var
    Mask: Byte;
    B: PByteArray;
    X, Y, Left, Right: Integer;
    ScanlineBytes, I: Integer;
    PR: PRect;
  begin
    B := BitmapMask.Scanline[0];
    ScanlineBytes := Integer(BitmapMask.Scanline[1]) - Integer(B);
    for Y := 0 to BitmapMask.Height-1 do
    begin
      Left := 0;
      Right := -1;
      I := 0;
      Mask := $80;
      for X := 0 to BitmapMask.Width-1 do
      begin
        if (Mask and B[I]) <> 0 then
        begin
          if Right >= Left then
          begin
            New(PR);
            PR^.Left := Left;
            PR^.Top := Y;
            PR^.Right := Right+1;
            PR^.Bottom := Y+1;
            RectList.Add(PR);
          end;
          Left := Right+2;
        end;
        Inc(Right);
        Mask := Mask shr 1;
        if Mask = 0 then
        begin
          Inc(I);
          Mask := $80;
        end;
      end;
      if Right >= Left then
      begin
        New(PR);
        PR^.Left := Left;
        PR^.Top := Y;
        PR^.Right := Right+1;
        PR^.Bottom := Y+1;
        RectList.Add(PR);
      end;
      Inc(Integer(B), ScanlineBytes);
    end;
  end;

begin
  RectList := TList.Create;
  BitmapMask := TBitmap.Create;
  try
    BitmapMask.Assign(Bitmap);
    BitmapMask.Mask(TransparentColor);
    BitmapMask.PixelFormat := pf1bit;
    FindRegionRectangles;
    Size := SizeOf(TRgnData) + (RectList.Count * SizeOf(TRect));
    GetMem(RgnData, Size);
    try
      with RgnData^.rdh do
      begin
        dwSize := SizeOf(TRgnDataHeader);
        iType := RDH_RECTANGLES;
        nCount := RectList.Count;
        nRgnSize := SizeOf(TRect);
        rcBound := Rect(0, 0, Bitmap.Width-1, Bitmap.Height-1);
      end;
      Rects := PRect(@RgnData^.Buffer);
      for I := 0 to RectList.Count-1 do
      begin
        Rects^ := PRect(RectList.Items[I])^;
        Dispose(PRect(RectList.Items[I]));
        Inc(Rects);
      end;
      Result := ExtCreateRegion(nil, Size, RgnData^)
    finally
      FreeMem(RgnData, Size);
    end;
  finally
    BitmapMask.Free;
    RectList.Free;
  end;
end;

function XYInMiniIcon(X, Y: Integer): boolean;
begin
  Result:=False;
  if (X>IconMiniArea.Left) and (X<IconMiniArea.Right)
    and (Y>IconMiniArea.Top) and (Y<IconMiniArea.Bottom) then
  begin
    Result:=True;
  end;
end;

function XYInCloseIcon(X, Y: Integer): boolean;
begin
  Result:=False;
  if (X>IconCloseArea.Left) and (X<IconCloseArea.Right)
    and (Y>IconCloseArea.Top) and (Y<IconCloseArea.Bottom) then
  begin
    Result:=True;
  end;
end;

function GetDeviceToToy(Device:TJvHidDevice;ProductName:string;
          Pid:Integer;Vid:Integer):boolean;
begin
end;

procedure SplitToBarPos(s:string;var bar,beat,tick,kind:Integer);
var
  sl:TStrings;
  t:string;
begin
  sl:=TStringList.Create;
  s:=StringReplace(s,':',',',[rfReplaceAll]);
  sl.CommaText:=s;
  t:=sl.Strings[0];
  bar:=StrToInt(t);
  t:=sl.Strings[1];
  beat:=StrToInt(t);
  t:=sl.Strings[2];
  tick:=StrToInt(t);
//  t:=sl.Strings[3];
//  kind:=StrToInt(t);
  sl.Free;
end;

end.
